/*
 * generateparameters.cxx
 *
 */

#include "generateparameters.h"

int main( int argc, char *argv[] )
{
	bool help = false;
	std::string file;
    const short NO_OPTION1 = -1;

	int mode = NO_OPTION1, scale =NO_OPTION1;
	double start = 0.1, end = 10, step = (double) NO_OPTION1;
	int feats = NO_OPTION1, samples = NO_OPTION1;

	int error = 0;
	const int EXIT_OK = error++;
	const int ARG_ERROR = error++;

	vtksys::CommandLineArguments arg;
	arg.AddBooleanArgument("--help", &help, "Give this output");
	arg.AddArgument("--mode", vtksys::CommandLineArguments::SPACE_ARGUMENT,
					&mode, "0: P \n "
							"\t\t 1: sigma \n"
							"\t\t 2: lambda ");
	arg.AddArgument("--file", vtksys::CommandLineArguments::SPACE_ARGUMENT,
					&file, "Output file");
	arg.AddArgument("--scale", vtksys::CommandLineArguments::SPACE_ARGUMENT,
						&scale, "0: Add \n \t\t 1: Multiply"
								"\n \t\t 2: Decimal\n"
								"\t\t 3: Log");
	arg.AddArgument("--start", vtksys::CommandLineArguments::SPACE_ARGUMENT,
						&start, "Start value");
	arg.AddArgument("--end", vtksys::CommandLineArguments::SPACE_ARGUMENT,
							&end, "End value");
	arg.AddArgument("--step", vtksys::CommandLineArguments::SPACE_ARGUMENT,
							&step, "Step value");
	arg.AddArgument("--feats", vtksys::CommandLineArguments::SPACE_ARGUMENT,
								&feats, "Number of features. Required for "
										"sigma generation");
	arg.AddArgument("--samples", vtksys::CommandLineArguments::SPACE_ARGUMENT,
								&samples, "Number of samples. Required for "
										"sigma and lambda generation");

	arg.Initialize(argc, argv);

	int result = 0;
	if (!(result = arg.Parse()) || help)
	{
		if (!result)
			std::cerr << "Error: problem parsing arguments" << std::endl;
		usage(arg.GetHelp(), ARG_ERROR);
	}

	if (mode == NO_OPTION1 || file == "" || scale == NO_OPTION1 || step == NO_OPTION1)
	{
		std::cerr << "Error: missing required arguments" << std::endl;
		usage(arg.GetHelp(), ARG_ERROR);
	}

	if (scale < 0 || scale > 3)
		scale = 0;

	if (mode >=1 && feats == NO_OPTION1)
	{
		std::cerr << "Error: missing required arguments for lambda/sigma"
			<< std::endl;
		usage(arg.GetHelp(), ARG_ERROR);
	}

	if (mode ==1 && samples == NO_OPTION1)
	{
		std::cerr << "Error: missing required arguments for sigma"
			<< std::endl;
		usage(arg.GetHelp(), ARG_ERROR);
	}

	std::vector<double> values;
	dld::ParametersGenerator* pg = new dld::ParametersGenerator();
	pg->setScale( scale );

	if (step <= 1 && scale == 1) // multiply mode step must be >1
	{
		std::cout << "Warning: Step must be > 1 for multiply scale. Value fixed "
				"to 2 " << std::endl;
		step = 2;
	}

	if (step <= 0 && (scale == 0 || scale == 3))
	{
		std::cout << "Warning: Step must be > 0 for add/log scale. Value fixed "
						"to 0.1 " << std::endl;
		step = 0.1;
	}

	if (step <= 1 && scale == 2)
	{
		std::cout << "Warning: Step must be an integer > 1 for decimal scale. "
				"Value fixed to 2 " << std::endl;
		step = 2;
	}

	switch (mode)
	{
	case 0:
	{
		values = pg->generatePFile(start,end,step);
		break;
	}
	case 1:
	{
		values = pg->generateSigmaFile(step,feats,samples);
		break;
	}
	case 2:
	{
		values = pg->generateLambdaFile(step,samples);
		break;
	}
	default:
		values = pg->generatePFile(0.01,0.99,0.01);
	}

	dld::FileWriter::writeVectorToFile<double>(file,values);

	return EXIT_OK;
}

void usage(const char *help, const int error)
{
	std::cerr << "Usage: generateparameters [args]" << std::endl;
	std::cerr << help;
	exit(error);
}
