/*
 * lpu-method.cxx
 *
 */

#include "lpu-method.h"



int main( int argc, char *argv[] )
{

	bool help = false;
	bool all = true;	//Extension parameter
	/** Flag for optimization on p (print only the best lambda/sigma) **/
	bool optimize = false;
	/** Positive samples file **/
	std::string positive_file;
	/** Unlabeled samples file **/
	std::string unlabeled_file;
	/** Output pattern model file**/
	std::string model_file;
	/** Output results (classified) data **/
	std::string output_file;
	/** p file **/
	std::string prop_file;
	/** lambda file **/
	std::string lambda_file;
	/** sigma file **/
	std::string sigma_file;

	int error = 0;
	const int EXIT_OK = error++;
	const int ARG_ERROR = error++;
	const int FILE_ERROR = error++;
	const int CONFIG_ERROR = error++;

	/** Config file reading **/
	std::string libsvm = LIBSVM_ROOT;

	vtksys::CommandLineArguments arg;
	arg.AddBooleanArgument("--help", &help, "Give this output");
	arg.AddBooleanArgument("--optimize", &optimize, "Show optimal value "
			"(lambda/sigma) per P value");
	arg.AddArgument("--positive", vtksys::CommandLineArguments::SPACE_ARGUMENT,
				&positive_file, "Positive samples file with format");
	arg.AddArgument("--unlabeled", vtksys::CommandLineArguments::SPACE_ARGUMENT,
					&unlabeled_file, "Unalabeled samples file with format");
	arg.AddArgument("--model", vtksys::CommandLineArguments::SPACE_ARGUMENT,
				&model_file, "model file");
	arg.AddArgument("--out", vtksys::CommandLineArguments::SPACE_ARGUMENT,
					&output_file, "Output results (classified) data file");
	arg.AddArgument("--props", vtksys::CommandLineArguments::SPACE_ARGUMENT,
						&prop_file, "P(1) value file");
	arg.AddArgument("--lambda", vtksys::CommandLineArguments::SPACE_ARGUMENT,
						&lambda_file, "lambda value file");
	arg.AddArgument("--sigma", vtksys::CommandLineArguments::SPACE_ARGUMENT,
							&sigma_file, "Sigma values (RBF Gaussian kernel) file");

	arg.Initialize(argc, argv);

	int result = 0;
	if (!(result = arg.Parse()) || help)
	{
		if (!result)
			std::cerr << "Error: problem parsing arguments" << std::endl;
		usage(arg.GetHelp(), ARG_ERROR);
	}

	if (!dld::FileReader::fileExists(libsvm + "/svm-train") &&
				!dld::FileReader::fileExists(libsvm + "/svm-predict"))
	{
		std::cerr << "Fatal Error: No LIBSVM detected" << std::endl;
		usage(arg.GetHelp(), ARG_ERROR);
	}
	else
		std::cout << "Using LIBSVM found at: " << libsvm << std::endl;

	//Arguments validation
	if (all)
	{
		if (positive_file == "" || unlabeled_file == "" || model_file == ""
			|| output_file == "" || prop_file == "" || lambda_file == ""
				|| sigma_file == "")
		{
			std::cerr << "Error: Missing required files" << std::endl;
			usage(arg.GetHelp(), ARG_ERROR);
		}

		if (!dld::FileReader::fileExists(positive_file) &&
				!dld::FileReader::fileExists(unlabeled_file) &&
				!dld::FileReader::fileExists(prop_file) &&
				!dld::FileReader::fileExists(lambda_file) &&
				!dld::FileReader::fileExists(sigma_file))
		{
			std::cerr << "Error: Required files do not exist" << std::endl;
			usage(arg.GetHelp(), FILE_ERROR);
		}

		dld::LPUAlgorithm* lpu = new dld::LPUAlgorithm();
		lpu->setPath(libsvm);
		lpu->setPositiveDataFile(positive_file);
		lpu->setUnlabeledDataFile(unlabeled_file);
		lpu->setModelFile(model_file);
		lpu->setOutputFile(output_file);
		lpu->setPFile(prop_file);
		lpu->setLambda(lambda_file);
		lpu->setSigmaFile(sigma_file);
		lpu->setOptimize(optimize);
		lpu->doAll();
	}

	return EXIT_OK;
}

void usage(const char *help, const int error)
{
	std::cerr << "Usage: lpu-algorithm [args]" << std::endl;
	std::cerr << help;
	exit(error);
}
