/*
 * dldLPUAlgorithm.h
 *
 */

#ifndef DLDLPUALGORITHM_H_
#define DLDLPUALGORITHM_H_

#define MERGE "lpufile"
#include <vector>
#include <string>

#include "dldTrainer.h"
#include "dldPredictor.h"
#include "dldUtils.h"
#include "dldFileReader.h"
#include "dldFileWriter.h"

#include <vtksys/SystemTools.hxx>

namespace dld
{

class LPUAlgorithm
{
public:

	/** Constructor **/
	LPUAlgorithm();

	/** Destructor **/
	~LPUAlgorithm();

	/** Sets P file **/
	void setPFile(const std::string p) { pValues = p; }

	/** Sets sigma file **/
	void setSigmaFile(const std::string s) { sValues = s; }

	/** Sets lambda file **/
	void setLambda(const std::string l) { lambdaValues = l; }

	/** Sets the model file **/
	void setModelFile(const std::string m);

	/** Sets the output file **/
	void setOutputFile(const std::string o);

	/** Sets the ground truth file **/
	void setGroundTruthFile(const std::string g) { groundtruth = g; }

	/** Sets the results file **/
	void setResultsFile(const std::string r) { results = r; }

	/** Sets the positive data file **/
	void setPositiveDataFile(const std::string p);

	/** Sets the unsampled data file **/
	void setUnlabeledDataFile(const std::string u);

	/** Performs a complete LPU routine: train, predict, obtain risk**/
	void doAll(void);

	/** Sets LIBSVM path. Useful in case of change **/
	void setPath(std::string p) { path = p; }

	void setOptimize(bool o) { optimizeModel = o; }

private:
	/** proporion vector **/
	std::vector<float> P;
	/** sigma (RBF Gaussian kernel) testing values **/
	std::vector<float> sigma;
	/** lambda values for training **/
	std::vector<float> lambda;
	/** Optimized risks (per p value) **/
	std::vector<float> risks;
	/** File containing all evaluated p values **/
	std::string pValues;
	/** File containing all evaluated lambda values **/
	std::string lambdaValues;
	/** File containing all evaluated sigma values **/
	std::string sValues;
	/** Output model file **/
	std::string model;
	/** Output model file extension**/
	std::string model_ext;
	/** Output classified data **/
	std::string output;
	/** Output classified data extension**/
	std::string output_ext;
	/** Statistical results file **/
	std::string results;
	/** Ground truth used for statistics **/
	std::string groundtruth;
	/** Input positive data **/
	std::string positivedata;
	/** Input unlabeled data **/
	std::string unlabeleddata;
	/** SVM sofware path **/
	std::string path;

	/** Number of positive samples **/
	unsigned int num_positive;
	/** Number of unlabeled data **/
	unsigned int num_labeled;

	/** Optimize per p flag**/
	bool optimizeModel;

	/** Trains under LPU **/
	void trainLPU(void) const;

	/** Prediction using trained models **/
	void predictLPU(void) const;

	/** Evaluation of risk using predictions **/
	float evaluateRisk(const std::string o, float p) const;

};

}

#endif /* DLDLPUALGORITHM_H_ */
