/*
 * dldParameterFileGenerator.cxx
 *
 */

#include "dldParametersGenerator.h"

namespace dld
{

/******************** P file ***********************************************/
std::vector<double> ParametersGenerator::generatePFile(
		const double start, const double end,
		const double step) const
{
	return generate(start,end,step);

}

/******************** Sigma file ******************************************/
std::vector<double> ParametersGenerator::generateSigmaFile(double
		step, const unsigned int numfeats, const unsigned int numsamples) const
{
	double start = 0.0001 * sqrt(numsamples) / numfeats;
	double end = 1000 * sqrt(numsamples) / numfeats;

	if (scale == 0)
		step *= (sqrt(numsamples) / numfeats);

	return generate(start,end,step);
}

/******************** Lambda file ******************************************/
std::vector<double> ParametersGenerator::generateLambdaFile(double
		step, const unsigned int numsamples) const
{
	double start = 0.00001 / sqrt(numsamples);
	double end = 1000 / sqrt(numsamples);

	if (scale == 0)
		step /= sqrt(numsamples);

	return generate(start,end,step);

}

/******************** Set scale *********************************************/
void ParametersGenerator::setScale(const unsigned int s)
{
	if (s < 0 || s > 3)
	{
		scale = 0;
		std::cerr << "Incorrect scale option. Setting it to default value Add."
			<< std::endl;
	}
	else
		scale = s;
}

/******************** generates values **************************************/
std::vector<double> ParametersGenerator::generate(double start,
		const double end, const double step) const
{
	std::vector<double> values;

	if (scale == 0)
	{
		values = addScale(start,end,step);
	}
	else if (scale == 1)
	{
		values = multiplyScale(start,end,step);
	}
	else if (scale == 2)
	{
		values = decimalScale(start,end,step);
	}
	else
	{
		values = logScale(start,end,step);
	}

	return values;
}

/******************** scale add up *******************************************/
std::vector<double> ParametersGenerator::addScale(double start,
		const double end, const double step) const
{
	std::vector<double> values;

	do
	{
		values.push_back(start);
		start +=step;
	} while (start <= end);

	return values;
}

/******************** scale multiply *****************************************/
std::vector<double> ParametersGenerator::multiplyScale(double start,
		const double end, const double step) const
{
	std::vector<double> values;
	do
	{
		values.push_back(start);
		start *=step;
	} while (start <= end);

	return values;
}

/******************** scale decimal *****************************************/
std::vector<double> ParametersGenerator::decimalScale(double start,
		const double end, const double step) const
{

	std::vector<double> values;

	values.push_back(start);
	while (start < end)
	{
		double temp_end = start * 10;
		double temp_step = temp_end / (step);
		for (unsigned short i = 1; i < step; ++i)
		{
			values.push_back(temp_step * i);
		}
		start = temp_end;
		values.push_back(start);
	}

	return values;
}
/******************** scale log *****************************************/
std::vector<double> ParametersGenerator::logScale(double start,
		const double end, const double step) const
{
	std::vector<double> values;
	unsigned int i = 1;
	double init_val = start;
	do
	{
		values.push_back((start));
		start =(init_val)*exp(i * step);
		i++;
	} while (start <= end);

	return values;
}

}//end namespace

