/*
 * dldParameterFileGenerator.h
 *
 */

#ifndef DLDPARAMETERFILEGENERATOR_H_
#define DLDPARAMETERFILEGENERATOR_H_

#include <vector>
#include <iostream>

#include "stdio.h"
#include "math.h"
#include "stdlib.h"

namespace dld
{

class ParametersGenerator
{
public:

	ParametersGenerator() { scale = 0; }
	/**
	 * This function generates the density range file. start and end values,
	 * as well as step, must be provided.
	 */
	std::vector<double> generatePFile(const double start,
			const double end, const double step) const;

	/**
	 * This function generates the sigma range file for RBF Gaussian kernel.
	 * Desired step, number of samples and features are required.
	 */
	std::vector<double> generateSigmaFile(double step,
			const unsigned int numfeats, const unsigned int numsamples) const;

	/**
	 * This function generates the lambda range file (regularization term).
	 * Desired step and number of samples are required.
	 */
	std::vector<double> generateLambdaFile(double step,
			const unsigned int numsamples) const;

	/**
	 * Sets the desired incremental scale for the parameters. Options are:
	 * 0: Add (+ step), 1: Multiply (*step), 2: Decimal and 3: Logarithmically.
	 * When 2 or 3 are set, the step value is ignored in the generateXXX methods
	 */
	void setScale(const unsigned int s = 0);

	//dld-svm used
	//std::vector<double> generateCFile(double step = 0.1);

private:
	unsigned int scale;

	/** Executes Add scale **/
	std::vector<double> addScale(double start, const double end,
			const double step) const;

	/** Executes multiply scale **/
	std::vector<double> multiplyScale(double start, const double end,
				const double step) const;

	/** Executes decimal scale **/
	std::vector<double> decimalScale(double start, const double end,
					const double step) const;

	/** Executes log scale **/
	std::vector<double> logScale(double start, const double end,
					const double step) const;

	/** Common interface for the generation of values **/
	std::vector<double> generate(double start,
			const double end, const double step) const;

};
}

#endif /* DLDPARAMETERFILEGENERATOR_H_ */
