/*
 * dldFileWriter.h
 *
 */

#ifndef DLDFILEWRITER_H_
#define DLDFILEWRITER_H_

#include <string>
#include <iostream>
#include <fstream>
#include <vector>
#include <ios>

#include "dldUtils.h"

namespace dld
{

class FileWriter
{
public:
	/** Merges two files fileone, filetwo into output file**/
	static void mergeFiles(const std::string fileone, const std::string filetwo,
			const std::string output);

	/** Deletes a file **/
	static void deleteFile(const std::string filename);

	/**Writes vector into a file. Should be templated **/
	template<typename T>
	static void writeVectorToFile(const std::string filename,
			const std::vector<T> v);

	/**Writes vector into a file using a line format. Should be templated **/
	template<typename T>
	static void writeVectorLineToFile(const std::string filename,
				const std::vector<T> v, bool append = false, bool count = false,
				bool firstislabel = false);

	static void writeStringVectorLineToFile(const std::string filename,
			const std::vector<std::string> v, bool append = false,
			bool count = false);
};

template<typename T>
void FileWriter::writeVectorToFile(const std::string filename,
			const std::vector<T> v)
{
	std::ofstream file;

	file.open( filename.c_str() );

	for (unsigned int i = 0; i < v.size(); ++i)
	{
		file << v[i] << std::endl;
	}

	file.close();
}


/**Writes vector into a file using a line format. Should be templated **/
template<typename T>
void FileWriter::writeVectorLineToFile(const std::string filename,
			const std::vector<T> v, bool append, bool count,
			bool firstislabel)
{
	std::ofstream file;

	if (append)
	{
		file.open( filename.c_str(), std::ios::app );
	}
	else
	{
		file.open( filename.c_str() );
	}


	for (unsigned int i = 0; i < v.size(); ++i)
	{
		if (firstislabel && i == 0 && v[0] == 1)
			file << "+";
		else if (count && i != 0)
			file << i << ":";

		file << v[i] << " ";
	}
	file << std::endl;
	file.close();
}

}
#endif /* DLDFILEWRITER_H_ */
